public static void test_filter()
{
  Test.add_func("/filter/boolean", () => {
    Notification message = new Notification.from_data("Message Application",
      "message-icon",
      "Message Summary",
      "Message Body");

    PropertyFilter filter_1 = new PropertyFilter(PropertyFilter.PropertyType.APP_NAME, "Message Application");
    PropertyFilter filter_2 = new PropertyFilter(PropertyFilter.PropertyType.BODY, "Message Body");
    PropertyFilter filter_3 = new PropertyFilter(PropertyFilter.PropertyType.SUMMARY, "Not Message Summary");

    AndFilterGroup and_group = new AndFilterGroup();

    /* An empty and group is considered true */
    assert(and_group.match(message));

    and_group.add_filter(filter_1);

    assert(and_group.match(message));

    and_group.add_filter(filter_2);

    assert(and_group.match(message));

    and_group.add_filter(filter_3);
    
    assert(!and_group.match(message));

    OrFilterGroup or_group = new OrFilterGroup();

    /* An empty or group is considered false */
    assert(!or_group.match(message));

    or_group.add_filter(filter_3);

    assert(!or_group.match(message));

    or_group.add_filter(filter_2);

    assert(or_group.match(message));

    or_group.add_filter(filter_1);

    assert(or_group.match(message));
  });

  Test.add_func("/filter/empty", () => {
    Notification message_1 = new Notification.from_data("Message Application",
      "message-icon",
      "   ",
      "  ");
    Notification message_2 = new Notification.from_data("Message Semi-Empty Application",
      "semi-empty-icon",
      "Has a summary but no body",
      "");

    EmptyFilter filter_1 = new EmptyFilter();

    assert(filter_1.match(message_1));
    assert(!filter_1.match(message_2));
  });

  Test.add_func("/filter/hash", () => {
    Notification message_1 = new Notification.from_data("Gwibber Social Portal",
      "gwibber-social-portal",
      "CheeseyBeard",
      "Arr, ye matey.");
    Notification message_2 = new Notification.from_data("Pidgin Instant Messenger",
      "pidgin-instant-messenger",
      "Somebody you know",
      "Has arrived.");
    Notification message_3 = new Notification.from_data("Some other application",
      "other-app",
      "You can see me",
      "I am right here.");

    HashPropertyFilter filter_1 = new HashPropertyFilter(PropertyFilter.PropertyType.APP_NAME);

    string[] values = {"Gwibber Social Portal", "Pidgin Instant Messenger", "Planet Ubuntu"};
    
    filter_1.set_values(values);

    assert(filter_1.match(message_1));
    assert(filter_1.match(message_2));
    assert(!filter_1.match(message_3));
  });

  Test.add_func("/filter/property", () => {
    Notification sample_1 = new Notification.from_data("Sample 1", 
        "sample-1-icon", 
        "Sample Summary 1", 
        "This is the body of the first sample message.");
    Notification sample_2 = new Notification.from_data("Sample 2", 
        "sample-2-icon", 
        "Sample Summary 2", 
        "This body is slightly different for sample 2.");

    PropertyFilter filter = new PropertyFilter(PropertyFilter.PropertyType.APP_NAME, "Sample 1");
    assert(filter.match(sample_1));
    assert(!filter.match(sample_2));

    filter = new PropertyFilter(PropertyFilter.PropertyType.APP_ICON, "sample-2-icon");
    assert(filter.match(sample_2));
    assert(!filter.match(sample_1));
  });

  Test.add_func("/filter/regex", () => {
    Notification sample_1 = new Notification.from_data("Gwibber Social Client",
      "gwibber",
      "Some Hollywood Guy",
      "I had eggs for breakfast, yummo!");
    Notification sample_2 = new Notification.from_data("Pidgin Internet Messenger", 
      "pidgin", 
      "Your Friend", 
      "Want to go see a movie?");

    RegexPropertyFilter filter_1 = new RegexPropertyFilter(PropertyFilter.PropertyType.APP_NAME,
      "^Gwibber");
    RegexPropertyFilter filter_2 = new RegexPropertyFilter(PropertyFilter.PropertyType.BODY,
      "see a movie");

    assert(filter_1.match(sample_1));
    assert(!filter_1.match(sample_2));
    assert(filter_2.match(sample_2));
    assert(!filter_2.match(sample_1));
  });
}
